import string

import log
import logging

import os



logger = logging.getLogger('Formatter')


class RubyFolderVerify:
    def __init__(self):
        self.type_of_file = '.rb'

    def check_name_of_folder(self, full_name_of_folder):
        name_of_folder = RubyFolderVerify._get_only_name_of_folder(full_name_of_folder)
        name_of_folder = self._check_snake_case_format(name_of_folder, isFile=False)

    def _check_snake_case_format(self, name_of_folder, isFile):
        logger.debug(f'verify {name_of_folder}')

        self.list_char_name = RubyFolderVerify._get_chars_from_str(name_of_folder)

        recommend_new_name = ''

        for i in range(len(self.list_char_name)):

            if self.list_char_name[i].isnumeric():
                recommend_new_name += self.list_char_name[i]

            elif self.list_char_name[i].isalpha():

                if self.list_char_name[i].isupper():
                    recommend_new_name += self._verify_big_letter(i)
                else:
                    recommend_new_name += self.list_char_name[i]

            elif self.list_char_name[i] == '_':
                recommend_new_name += self._verify_under_line(i)

        if isFile:
            name_of_folder += self.type_of_file
            recommend_new_name += self.type_of_file

        if recommend_new_name != name_of_folder:
            logger.debug(f'You should name {name_of_folder} to {recommend_new_name}')
        else:
            logger.debug(f'name {name_of_folder} is ok')


        return recommend_new_name

    def _verify_big_letter(self, index):
        recommend_new_name_of_folder = ''
        logger.debug(f'find upper letter --> {self.list_char_name[index]}')
        if index > 0:
            if self.list_char_name[index - 1] != '_':
                recommend_new_name_of_folder += '_'
            recommend_new_name_of_folder += self.list_char_name[index].lower()
            return recommend_new_name_of_folder
        else:
            return self.list_char_name[index].lower()

    def _verify_under_line(self, index):
        recommend_new_name_of_folder = ''
        if index <= len(self.list_char_name):

            if self.list_char_name[index + 1].isnumeric():
                logger.debug(f'find number after {self.list_char_name[index]}')
            else:
                recommend_new_name_of_folder += self.list_char_name[index]
        return recommend_new_name_of_folder

    @staticmethod
    def _get_only_name_of_folder(full_name_of_folder):
        list_chars = RubyFolderVerify._get_chars_from_str(full_name_of_folder)
        list_name_of_folder = []

        i = len(list_chars) - 1
        while list_chars[i] != '/':
            list_name_of_folder.append(list_chars[i])
            i -= 1

        list_name_of_folder.reverse()
        return RubyFolderVerify._get_str_from_chars(list_name_of_folder)

    @staticmethod
    def _get_only_name_of_file(full_name_of_folder):
        list_chars = RubyFolderVerify._get_chars_from_str(full_name_of_folder)
        list_name_of_folder = []

        j = 0
        i = len(list_chars) - 1
        while list_chars[i] != '.':
            i -= 1
            j += 1
        j += 1

        i = len(list_chars) - 1 - j
        while list_chars[i] != '/':
            # print(list_chars[i])
            list_name_of_folder.append(list_chars[i])
            i -= 1

        list_name_of_folder.reverse()
        return RubyFolderVerify._get_str_from_chars(list_name_of_folder)

    @staticmethod
    def _get_chars_from_str(str):
        return [char for char in str]

    @staticmethod
    def _get_str_from_chars(chars_list):
        str = ""

        for char in chars_list:
            str += char

        return str

    @staticmethod
    def _convert_list_to_string(org_list, seperator='\n'):
        return seperator.join(org_list)



import logging

from .fileFormatter import FileFormatter
from .cmdParser import CmdParser

logger = logging.getLogger('Formatter')

logger.debug(f'program started')


def run():
    FileFormatter(CmdParser.parse_arguments()).run()


run()

# PositionConfig('default.editorconfig')

logger.debug(f'program finished')

#
# import lab2FmtLipsky
#
# lab2FmtLipsky.FileFormatter(lab2FmtLipsky.CmdParser.parse_arguments()).run()



import argparse
from enum import Enum
import os
import shutil

import logging
import log
import io

from cmdParser import CmdParser
from analizer import Analizer
from rubyCodeParser import RubyCodeParser
from rubyFolderVerify import RubyFolderVerify


import rubyCodeParser

logger = logging.getLogger('Formatter')


class FileFormatter:
    def __init__(self, dict):
        self.dict = dict
        self.need_to_fix = False

    @staticmethod
    def is_file_type_supported(file, needed_type=".rb"):
        file_name, type_name = os.path.splitext(file)

        if type_name == needed_type:
            logger.debug(f'{file} is suitable')
            return True
        else:
            logger.debug(f'{file} is NOT suitable')
            return False

    @staticmethod
    def create_formatted_file(file, formatted_str):
        file_name, type_name = os.path.splitext(file)
        formatted_file = file_name + '_formatted' + type_name

        text_file = open(formatted_file, "wt")
        n = text_file.write(formatted_str)
        text_file.close()

        logger.debug(f'created formatted file: {formatted_file}')

    def formatting(self, file_name):
        logger.debug(f'formatter works with {file_name}')

        if FileFormatter.is_file_type_supported(file_name):
            logger.debug(f'formatter is checking {file_name}')

            code_parser = RubyCodeParser(file_name)
            code_parser.check_name_of_file(file_name)

            formatted_str = code_parser.parser()

            if self.need_to_fix:
                FileFormatter.create_formatted_file(file_name, formatted_str)

        logger.debug(f'formatter finished work with {file_name} and went to the next')

    def run(self):
        list = Analizer.define_item(self.dict)

        if list[3] == 'True':
            self.need_to_fix = True

        if list[0] == 'project':
            list_of_directories = Analizer.come_across_project(list)

            for i in range(len(list_of_directories)):
                list_pro = ['', list_of_directories[i]]
                RubyFolderVerify().check_name_of_folder(list_of_directories[i])
                for file in Analizer.come_across_directory(list_pro):
                    self.formatting(file)

        if list[0] == 'directory':
            RubyFolderVerify().check_name_of_folder(list[1])

            for file in Analizer.come_across_directory(list):
                self.formatting(file)

        elif list[0] == 'file':
            self.formatting(list[1])







import logging

"""
Make notes by logging
"""

logger = logging.getLogger('Formatter')
logger.setLevel(logging.DEBUG)

formatter = logging.Formatter(fmt='%(asctime)s - %(name)s - %(levelname)s - %(filename)s:%(lineno)s - %(funcName)2s() - %(message)s')

file_handler = logging.FileHandler(filename='./logs/all.log')
file_handler.setLevel(logging.DEBUG)
file_handler.setFormatter(formatter)

console_handler = logging.StreamHandler()
console_handler.setLevel(logging.DEBUG)
console_handler.setFormatter(formatter)

logger.addHandler(file_handler)
# logger.addHandler(console_handler) # print logs to console


# logging.basicConfig(filename='./logs/all.log', filemode='a', level=logging.DEBUG,
#                         format='%(asctime)s - %(name)s - %(levelname)s - %(filename)s:%(lineno)s - %(funcName)2s() - %(message)s')
# logger = logging.getLogger('Lab1V7')



# def logging_set_up():
#     logging.basicConfig(filename='./logs/all.log', filemode='a', level=logging.DEBUG,
#                         format='%(asctime)s - %(name)s - %(levelname)s - %(filename)s:%(lineno)s - %(funcName)2s() - %(message)s')
#     logger = logging.getLogger('Lab1V7')
#
#     return logger